@@ -24,7 +24,7 @@
 #include <vmm_types.h>
 #include <vmm_error.h>
 #include <vmm_host_aspace.h>
-#include <serial/uart.h>
+#include <drv/uart.h>
 
 static virtual_addr_t uart_base = 0;
 extern virtual_addr_t isa_vbase;
@@ -54,7 +54,7 @@ int arch_defterm_putc(u8 ch)
 int arch_defterm_init(void)
 {
 	uart_base = isa_vbase + 0x3F8;
-	uart_lowlevel_init("ns8250", uart_base, 1, 115200, 1843200);
+	uart_lowlevel_init(uart_base, 1, 115200, 1843200);
 
 	return VMM_OK;
 }
