@@ -36,9 +36,23 @@ u32 arch_vcpu_irq_priority(struct vmm_vcpu * vcpu, u32 irq_no)
 	return 1;
 }
 
+/* FIXME: */
+int arch_vcpu_irq_assert(struct vmm_vcpu * vcpu, u32 irq_no, u32 reason)
+{
+	return VMM_OK;
+}
+
+/* FIXME: */
 int arch_vcpu_irq_execute(struct vmm_vcpu * vcpu,
 			 arch_regs_t * regs, 
 			 u32 irq_no, u32 reason)
 {
 	return VMM_OK;
 }
+
+/* FIXME: */
+int arch_vcpu_irq_deassert(struct vmm_vcpu * vcpu, u32 irq_no, u32 reason)
+{
+	return VMM_OK;
+}
+
