@@ -22,42 +22,69 @@
  */
 
 #include <vmm_error.h>
-#include <vmm_list.h>
-#include <vmm_string.h>
+#include <vmm_compiler.h>
 #include <vmm_heap.h>
+#include <vmm_scheduler.h>
 #include <vmm_chardev.h>
+#include <libs/stringlib.h>
 
-int vmm_chardev_doioctl(struct vmm_chardev * cdev,
-			int cmd, void *buf, size_t len, bool block)
+int vmm_chardev_doioctl(struct vmm_chardev *cdev,
+			int cmd, void *buf, u32 len)
 {
 	if (cdev && cdev->ioctl) {
-		return cdev->ioctl(cdev, cmd, buf, len, block);
+		return cdev->ioctl(cdev, cmd, buf, len);
 	} else {
 		return VMM_EFAIL;
 	}
 }
 
-u32 vmm_chardev_doread(struct vmm_chardev * cdev,
-		       u8 *dest, size_t offset, size_t len, bool block)
+u32 vmm_chardev_doread(struct vmm_chardev *cdev,
+		       u8 *dest, u32 len, bool block)
 {
+	u32 b;
+	bool sleep;
+
 	if (cdev && cdev->read) {
-		return cdev->read(cdev, dest, offset, len, block);
+		if (block) {
+			b = 0;
+			sleep = vmm_scheduler_orphan_context() ? TRUE : FALSE;
+			while (b < len) {
+				b += cdev->read(cdev, &dest[b], 
+						len - b, sleep);
+			}
+			return b;
+		} else {
+			return cdev->read(cdev, dest, len, FALSE);
+		}
 	} else {
 		return 0;
 	}
 }
 
-u32 vmm_chardev_dowrite(struct vmm_chardev * cdev,
-			u8 *src, size_t offset, size_t len, bool block)
+u32 vmm_chardev_dowrite(struct vmm_chardev *cdev,
+			u8 *src, u32 len, bool block)
 {
+	u32 b;
+	bool sleep;
+
 	if (cdev && cdev->write) {
-		return cdev->write(cdev, src, offset, len, block);
+		if (block) {
+			b = 0;
+			sleep = vmm_scheduler_orphan_context() ? TRUE : FALSE;
+			while (b < len) {
+				b += cdev->write(cdev, &src[b], 
+						 len - b, sleep);
+			}
+			return b;
+		} else {
+			return cdev->write(cdev, src, len, FALSE);
+		}
 	} else {
 		return 0;
 	}
 }
 
-int vmm_chardev_register(struct vmm_chardev * cdev)
+int vmm_chardev_register(struct vmm_chardev *cdev)
 {
 	int rc;
 	struct vmm_classdev *cd;
@@ -71,10 +98,10 @@ int vmm_chardev_register(struct vmm_chardev * cdev)
 		return VMM_EFAIL;
 	}
 
-	vmm_memset(cd, 0, sizeof(struct vmm_classdev));
+	memset(cd, 0, sizeof(struct vmm_classdev));
 
 	INIT_LIST_HEAD(&cd->head);
-	vmm_strcpy(cd->name, cdev->name);
+	strcpy(cd->name, cdev->name);
 	cd->dev = cdev->dev;
 	cd->priv = cdev;
 
@@ -86,7 +113,7 @@ int vmm_chardev_register(struct vmm_chardev * cdev)
 	return rc;
 }
 
-int vmm_chardev_unregister(struct vmm_chardev * cdev)
+int vmm_chardev_unregister(struct vmm_chardev *cdev)
 {
 	int rc;
 	struct vmm_classdev *cd;
@@ -147,10 +174,10 @@ int __init vmm_chardev_init(void)
 		return VMM_EFAIL;
 	}
 
-	vmm_memset(c, 0, sizeof(struct vmm_class));
+	memset(c, 0, sizeof(struct vmm_class));
 
 	INIT_LIST_HEAD(&c->head);
-	vmm_strcpy(c->name, VMM_CHARDEV_CLASS_NAME);
+	strcpy(c->name, VMM_CHARDEV_CLASS_NAME);
 	INIT_LIST_HEAD(&c->classdev_list);
 
 	rc = vmm_devdrv_register_class(c);
